@@ -11,7 +11,7 @@ set -o pipefail
 
 display_help() {
   cat << EOF
-usage: debcraft <build|validate|test|release|shell|prune> [options] [<path|pkg|srcpkg|dsc|git-url>]
+usage: debcraft <build|improve|test|release|shell|prune> [options] [<path|pkg|srcpkg|dsc|git-url>]
 
 Debcraft is a tool to easily build .deb packages. The 'build' argument accepts
 as a subargument any of:
@@ -25,15 +25,18 @@ as a subargument any of:
 
   * git http(s) or ssh URL that can be downloaded and built
 
-The commands 'validate' and 'release' are intended to be used to finalize a
-package build. The command 'test' will run the Debian-specific regression test
+The command 'release' is intended to be used to upload a package that is ready
+to be released. The command 'test' will run the Debian-specific regression test
 suite if the package has autopkgtest support, and drop to a shell for
 investigation if tests failed to pass. The command 'shell' can be used to play
 around in the container and 'prune' will clean up temporary files by Debcraft.
 
 In addition to parameters below, anything passed in DEB_BUILD_OPTIONS will also
-be honored (currently DEB_BUILD_OPTIONS='$DEB_BUILD_OPTIONS'). Note that
-Debcraft builds never runs as root, and thus packages with
+be honored (currently DEB_BUILD_OPTIONS='$DEB_BUILD_OPTIONS'). Successful builds
+include running './debian/rules clean' to clean up artifacts, while failed
+builds will leave them around for inspection.
+
+Note that Debcraft builds never runs as root, and thus packages with
 DEB_RULES_REQUIRES_ROOT are not supported.
 
 optional arguments:
@@ -50,6 +53,8 @@ optional arguments:
   --pull               ensure container base is updated
   --copy               perform the build on a copy of the package directory
   --clean              ensure sources are clean before and after build
+                       (only needed on packages with incomplete 'debian/clean'
+                       or 'debian/.gitignore' definitions)
   --debug              emit debug information
   -h, --help           display this help and exit
   --version            display version and exit
@@ -118,7 +123,7 @@ source "$DEBCRAFT_LIB_DIR/generic.inc.sh"
 
 if [ -z "$1" ]
 then
-  log_error "Missing argument <build|validate|test|release|shell|prune>"
+  log_error "Missing argument <build|improve|test|release|shell|prune>"
   echo
   display_help
   exit 1
@@ -190,7 +195,7 @@ do
       ## or call function display_help
       exit 1
       ;;
-    build | validate | test | release | shell | prune)
+    build | improve | test | release | shell | prune)
       export ACTION="$1"
       shift
       ;;
@@ -205,7 +210,7 @@ done
 if [ -z "$ACTION" ]
 then
   # If ACTION is empty the TARGET might have been populated
-  log_error "Argument '$TARGET' not one of <build|validate|test|release|shell|prune>"
+  log_error "Argument '$TARGET' not one of <build|improve|test|release|shell|prune>"
   echo
   display_help
   exit 1
@@ -263,7 +268,7 @@ elif [ -d "$TARGET" ]
 then
   # If an argument was given, and it is a directory, use TARGET as-is
   :
-elif [ ! -d "$TARGET" ] && [[ "build validate" =~ $ACTION ]]
+elif [ ! -d "$TARGET" ] && [[ "build improve" =~ $ACTION ]]
 then
   # If the argument exists, but didn't point to a valid path, try to use the
   # argument to download the package
@@ -309,12 +314,8 @@ log_debug_var PWD
 
 if [ -f "debian/changelog" ]
 then
-  # If dpkg-parsechangelog fails and emits exit code, Debcraft will
-  # intentionally halt completely at this point
-  #
-  # @TODO: Having dpkg-parsechangelog as a dependency is against the design
-  # principle of having Debcraft as an universal tool
-  PACKAGE="$(dpkg-parsechangelog --show-field=source)"
+  # If parsing the changelog emits exit code, intentionally stop here
+  PACKAGE="$(head -n 1 debian/changelog | cut -d ' ' -f 1)"
 else
   log_error "Directory '$TARGET' is not a valid source package directory as" \
             "debian/changelog was not found"
@@ -329,9 +330,9 @@ then
   # Every deb build potentially generates these temporary files, and it is safe
   # to assume that they should be deleted before restarting the build, so do it
   # automatically to make the overall experience friendlier
-  rm -rf debian/.debhelper debian/debhelper-build-stamp \
-         debian/debhelper.log debian/*.debhelper.log \
-         debian/substvars debian/*.substvars debian/files
+  rm -rfv debian/.debhelper debian/debhelper-build-stamp \
+          debian/debhelper.log debian/*.debhelper.log \
+          debian/substvars debian/*.substvars debian/files
 
   # If there are still more files, the user needs to make a decision
   if [ -z "$CLEAN" ] &&
@@ -341,7 +342,10 @@ then
   then
     log_error "Modified or additional files found"\
               "\n$(git status --porcelain --ignored --untracked-files=all | head)"\
-              "\nCannot proceed building unless --clean or --copy is used."
+              "\nThese may be leftovers of a failed build, or of an incomplete"\
+              "\nrun of './debian/rules clean' after a successful build." \
+              "\nUse --clean to remove all untracked and uncommitted files"\
+              "\nor run with --copy to build in a separate new directory."
     exit 1
   fi
 fi
@@ -368,9 +372,9 @@ case "$ACTION" in
     # shellcheck source=src/build.inc.sh
     source "$DEBCRAFT_LIB_DIR/build.inc.sh"
     ;;
-  validate)
-    # shellcheck source=src/validate.inc.sh
-    source "$DEBCRAFT_LIB_DIR/validate.inc.sh"
+  improve)
+    # shellcheck source=src/improve.inc.sh
+    source "$DEBCRAFT_LIB_DIR/improve.inc.sh"
     ;;
   test)
     # shellcheck source=src/test.inc.sh
