@@ -18,11 +18,19 @@ case "$DISTRIBUTION" in
       fi
     else
       # Parse the latest debian/changelog entry
-      DISTRIBUTION="$(dpkg-parsechangelog  --show-field=distribution)"
+      DISTRIBUTION="$(head -n 1 debian/changelog | cut -d ' ' -f 3 | cut -d ';' -f 1)"
+      log_debug_var DISTRIBUTION
       # ..or if that is UNRELEASED, the second last entry
       if [ "$DISTRIBUTION" == "UNRELEASED" ]
       then
-        DISTRIBUTION="$(dpkg-parsechangelog  --show-field=distribution --offset=1 --count=1)"
+        DISTRIBUTION="$(grep --max-count=2 '^[^ ]\+ ([^)]\+) [^ ]\+;' debian/changelog | tail -n 1 | cut -d ' ' -f 3 | cut -d ';' -f 1)"
+        # If there is no second last entry, or if it too is UNRELEASED
+        # (unlikely), clear the variable from any value to let it fall-back to
+        # default values in a later step
+        if [ "$DISTRIBUTION" == "UNRELEASED" ]
+        then
+          DISTRIBUTION=""
+        fi
       fi
 
       # If really on UNRELEASED was only option, default to debian:sid, which
@@ -31,7 +39,7 @@ case "$DISTRIBUTION" in
       then
         BASEIMAGE="debian:sid"
       else
-        # Let function map dpkg-parsechangelog value to a sensible baseimage
+        # Let function map the extracted distribution name to a sensible baseimage
         BASEIMAGE="$(get_baseimage_from_distribution_name "$DISTRIBUTION")"
       fi
     fi
@@ -61,8 +69,10 @@ if [ -d "$PWD/.git" ]
 then
   # Set git commit id and name for later use
   COMMIT_ID="$(git -C "$PWD/.git" rev-parse --short HEAD)"
-  # Strip branch paths and any slashes so version string is clean
-  BRANCH_NAME="$(git -C "$PWD/.git" symbolic-ref HEAD | sed 's|.*heads/||')"
+  # Strip branch paths and any slashes so version string is clean, and if there
+  # is no symbolic-ref at all ("fatal: ref HEAD is not a symbolic ref") when
+  # for example building a detached head, fall-back to using just commit id
+  BRANCH_NAME="$(git -C "$PWD/.git" symbolic-ref HEAD | sed 's|.*heads/||' || echo "$COMMIT_ID")"
 
   # The BUILD_ID will appended to the Debian/Ubuntu version string, and thus
   # cannot have slahses, dashes or underscores.
@@ -92,8 +102,17 @@ fi
 
 case "$BUILD_DIRS_PATH" in
   "")
-    # If BUILD_DIRS_PATH is not set, use use parent directory of TARGET
-    BUILD_DIRS_PATH="$(cd .. && pwd)"
+    # If BUILD_DIRS_PATH is not set, use $XDG_CACHE_HOME following
+    # https://specifications.freedesktop.org/basedir-spec/latest/
+    # or fall-back to ~/.cache as last resort
+    if [ -n "$XDG_CACHE_HOME" ]
+    then
+      BUILD_DIRS_PATH="$XDG_CACHE_HOME/debcraft"
+    else
+      BUILD_DIRS_PATH="$HOME/.cache/debcraft"
+    fi
+    # Ensure directory exists
+    mkdir -p "$BUILD_DIRS_PATH"
     ;;
   *)
     # If BUILD_DIRS_PATH is defined, use it as-is
@@ -118,10 +137,12 @@ else
   exit 1
 fi
 
-# Actions 'build' and 'validate' operate on the BUILD_DIR
+# Actions 'build' and 'improve' operate on the BUILD_DIR
 BUILD_DIR="$BUILD_DIRS_PATH/debcraft-build-$PACKAGE-$BUILD_ID"
 # Action 'release' places artifacts in a differently named directory
 RELEASE_DIR="$BUILD_DIRS_PATH/debcraft-release-$PACKAGE-$BUILD_ID"
+# Shared cache directory for all builds of the same package
+CACHE_DIR="$BUILD_DIRS_PATH/debcraft-cache-$PACKAGE"
 
 # Explicit exports
 export PACKAGE
@@ -131,6 +152,7 @@ export BUILD_ID
 export BUILD_DIRS_PATH
 export BUILD_DIR
 export RELEASE_DIR
+export CACHE_DIR
 
 log_info "Use '$CONTAINER_CMD' container image '$CONTAINER' for package '$PACKAGE'"
 
@@ -169,7 +191,8 @@ export PREVIOUS_SUCCESSFUL_RELEASE_DIRS
 # latest tagged releases
 if [ -d "$PWD/.git" ]
 then
-  if [ -n "$(git tag --list)" ]
+  # We don't want to have import tags such as "upstream/*" but only release tags
+  if [ -n "$(git tag --list "debian/*" "ubuntu/*")" ] && git describe --first-parent --abbrev=0 > /dev/null
   then
     # Get the first tag encountered when traversing branch history
     NAME_OF_LAST_TAG="$(git describe --first-parent --abbrev=0)"
