@@ -24,14 +24,14 @@ if [ -n "${PREVIOUS_SUCCESSFUL_RELEASE_DIRS[0]}" ]
 then
   log_info "Previous release was in ${PREVIOUS_SUCCESSFUL_RELEASE_DIRS[0]}"
   mkdir --parents "$RELEASE_DIR/previous"
-  CONTAINER_RUN_ARGS=" --volume=${PREVIOUS_SUCCESSFUL_RELEASE_DIRS[0]}:/tmp/build/previous $CONTAINER_RUN_ARGS"
+  CONTAINER_RUN_ARGS+=("--volume=${PREVIOUS_SUCCESSFUL_RELEASE_DIRS[0]}:/debcraft/previous")
 fi
 
 if [ -n "${LAST_TAGGED_SUCCESSFUL_RELEASE_DIRS[0]}" ]
 then
   log_info "Previous tagged release was in ${LAST_TAGGED_SUCCESSFUL_RELEASE_DIRS[0]}"
   mkdir --parents "$RELEASE_DIR/last-tagged"
-  CONTAINER_RUN_ARGS=" --volume=${LAST_TAGGED_SUCCESSFUL_RELEASE_DIRS[0]}:/tmp/build/last-tagged $CONTAINER_RUN_ARGS"
+  CONTAINER_RUN_ARGS+=("--volume=${LAST_TAGGED_SUCCESSFUL_RELEASE_DIRS[0]}:/debcraft/last-tagged")
 fi
 
 if [ -n "$WITH_BINARIES" ] || [ -n "$DEBCRAFT_FULL_BUILD" ]
@@ -42,14 +42,14 @@ else
   log_info "Building source package for release at $RELEASE_DIR"
 fi
 
-# Define variable only used in build
-CCACHE_DIR="$BUILD_DIRS_PATH/ccache"
-mkdir --parents "$CCACHE_DIR" "$RELEASE_DIR/source"
+# Ensure directories exist before they are mounted
+mkdir --parents "$CACHE_DIR" "$BUILD_DIR/source"
+
 # Instead of plain 'chown -R' use find and only apply chmod on files that need
 # it to avoid excess disk writes and ctime updates in vain. Use 'execdir' as
 # safer option to 'exec' and use the variant ending with plus so any non-zero
 # exit code will be surfaced and calling script aborted.
-find "$CCACHE_DIR" ! -uid "${UID}" -execdir chown --no-dereference --verbose "${UID}":"${GROUPS[0]}" {} +
+find "$CACHE_DIR" ! -uid "${UID}" -execdir chown --no-dereference --verbose "${UID}":"${GROUPS[0]}" {} +
 
 if [ -n "$DEBUG" ]
 then
@@ -60,16 +60,17 @@ fi
 # shellcheck disable=SC2086
 $CONTAINER_CMD run \
     --name="$CONTAINER" \
-    --interactive --tty --rm \
+    --interactive \
+    ${CONTAINER_CAN_HAVE_TTY:+--tty} \
+    --rm \
     --shm-size=1G \
     --network=none \
-    --volume="$CCACHE_DIR":/.ccache \
-    --volume="$RELEASE_DIR":/tmp/build \
-    --volume="${SOURCE_DIR:=$PWD}":/tmp/build/source \
-    --workdir=/tmp/build/source \
-    --env="CCACHE_DIR=/.ccache" \
+    --volume="$CACHE_DIR":/debcraft/cache \
+    --volume="$RELEASE_DIR":/debcraft \
+    --volume="${SOURCE_DIR:=$PWD}":/debcraft/source \
+    --workdir=/debcraft/source \
     --env="DEB*" \
-    $CONTAINER_RUN_ARGS \
+    "${CONTAINER_RUN_ARGS[@]}" \
     "$CONTAINER" \
     /debcraft-releaser.sh \
     || FAILURE="true"
@@ -93,8 +94,18 @@ then
   rmdir "$RELEASE_DIR/last-tagged"
 fi
 
-# Clean up temporary mount directory from polluting build artifacts
-rmdir "$RELEASE_DIR/source"
+
+# Clean up temporary mount directorie from polluting build artifacts, but
+# only if they exist (they might not exists when running inside DinD in CI)
+if [ -d "$RELEASE_DIR/source" ]
+then
+  rmdir "$RELEASE_DIR/source"
+fi
+
+if [ -d "$RELEASE_DIR/cache" ]
+then
+  rmdir "$RELEASE_DIR/cache"
+fi
 
 # If the container returned an error code, stop here after cleanup completed
 if [ -n "$FAILURE" ]
